/*
 * 2D Game Engine
 * Game.cpp: Base Game engine class
 * Based on pikuma.com 2D game engine in C++ and Lua course
 * Copyright (c) 2021 986-Studio. All Right Reserved
 *
 * Created by Manoël Trapier on 09/02/2021.
 */
#include <Game.h>
#include <stdio.h>
#include <SDL.h>

Game::Game()
{
    this->isRunning = false;
    this->windowsHeight = 0;
    this->windowsWidth = 0;
}

Game::~Game()
{

}

void Game::Initialize()
{
    int ret;
    SDL_DisplayMode displayMode;

    ret = SDL_Init(SDL_INIT_EVERYTHING);
    if (ret)
    {
        fprintf(stderr, "SDL Initialisation error.\n");
        return;
    }

    SDL_GetCurrentDisplayMode(0, &displayMode);
    this->windowsHeight = displayMode.h;
    this->windowsWidth = displayMode.w;
    printf("%d:%d\n", displayMode.w, displayMode.h);
    this->window = SDL_CreateWindow(NULL, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
                                    this->windowsWidth, this->windowsHeight, SDL_WINDOW_BORDERLESS);
    if (!window)
    {
        fprintf(stderr, "SDL window creation error.\n");
        return;
    }

    this->renderer = SDL_CreateRenderer(window, -1, 0);
    if (!renderer)
    {
        fprintf(stderr, "SDL renderer creation error.\n");
        return;
    }

    SDL_SetWindowFullscreen(this->window, SDL_WINDOW_FULLSCREEN);

    this->isRunning = true;
}

void Game::Destroy()
{
    if (this->renderer)
    {
        SDL_DestroyRenderer(this->renderer);
    }
    if (this->window)
    {
        SDL_DestroyWindow(this->window);
    }

    SDL_Quit();
}

void Game::Run()
{
    while(this->isRunning)
    {
        this->ProcessInput();
        this->Update();
        this->Render();
    }
}

void Game::ProcessInput()
{
    SDL_Event event;

    while(SDL_PollEvent(&event))
    {
        switch(event.type)
        {
            case SDL_QUIT:
                this->isRunning = false;
                break;

            case SDL_KEYDOWN:
                if (event.key.keysym.sym == SDLK_ESCAPE)
                {
                    this->isRunning = false;
                }
                break;

            default:
                break;
        }
    }

}

void Game::Update()
{

}

void Game::Render()
{
    SDL_SetRenderDrawColor(this->renderer, 255, 64, 127, 255);
    SDL_RenderClear(this->renderer);

    SDL_RenderPresent(this->renderer);
}
